part of simulator_eurl_sasu;


ServiceSimulationEurl serviceSimulationEurl = new ServiceSimulationEurl();

class ServiceSimulationEurl {

  Logger logger = new Logger('simulator.ServiceSimulationEurl');

  Map<double, double> findNetsByPercentage(double turnover, double runningCost) {

    Map<double, double> netByPercentage = new Map();

    for (var i = 0; i <= 100; i++) {
      SimulationEurl simulationEurl = computeSimulationPercentage(turnover, runningCost, i * 1.0);
      netByPercentage[i] = simulationEurl.net;
    }
    return netByPercentage;
  }

  SimulationEurl findBestSimulation(double turnover, double runningCost) {

    SimulationEurl bestSimulation = computeSimulationPercentage(turnover, runningCost, 0.00);
    for (var i = 1; i <= 100; i++) {
      SimulationEurl simulationEurl = computeSimulationPercentage(turnover, runningCost, i * 1.0);
      if(simulationEurl != null && simulationEurl.net > bestSimulation.net) {
        bestSimulation = simulationEurl;
      }
    }
    return bestSimulation;
  }

  SimulationEurl computeSimulationPercentage(double turnover, double runningCost, double remunerationPercentage) {

    if(remunerationPercentage < 0) {
      remunerationPercentage = 0.00;
    }

    if(remunerationPercentage > 100) {
      remunerationPercentage = 100.00;
    }

    double maxRemuneration = turnover - runningCost;
    return serviceUtil.findByDichotomy(100, 0.00, maxRemuneration, remunerationPercentage, (newLimit) => computeSimulationEmpiric(turnover, runningCost, newLimit), (_) => true, (computed) => (computed.remuneration.gross / maxRemuneration) * 100);
  }

  SimulationEurl computeSimulationGross(double turnover, double runningCost, double remunerationGross) {

    return serviceUtil.findByDichotomy(100, 0.00, remunerationGross, remunerationGross, (newLimit) => computeSimulationEmpiric(turnover, runningCost, newLimit), (_) => true, (computed) => computed.remuneration.gross, proportional: true);
  }

  SimulationEurl computeSimulationEmpiric(double turnover, double runningCost, double remuneration) {

    double maxDividend = turnover - runningCost - remuneration;
    return serviceUtil.findByDichotomy(100, 0.00, maxDividend, 0.00, (newLimit) => computeSimulation(turnover, runningCost, remuneration, newLimit), (computed) => (computed != null && computed.treasury > 0.00), (computed) => computed.treasury, proportional: false);
  }

  SimulationEurl computeSimulation(double turnover, double runningCost, double remuneration, double dividendGross) {

    Remuneration remunerationEurl = serviceRemuneration.computeRemuneration(remuneration, remuneration + dividendGross);

    double profit = turnover - runningCost - remunerationEurl.gross;

    double taxIs = serviceIrIs.computeIs(profit);
    Dividend dividend = serviceDividend.computeDividend(dividendGross);
    double ir = serviceIrIs.computeIr(remunerationEurl.netTaxable + dividend.netTaxable);

    double treasury = turnover - runningCost - remunerationEurl.gross - taxIs - dividendGross;

    double net = remunerationEurl.remuneration + dividend.net - ir;

    return new SimulationEurl(turnover, runningCost, remunerationEurl, taxIs, dividend, ir, net, treasury);
  }
}

